@@ -45,14 +45,20 @@ func newSource(sourceInfo *pulse.Source, audio *Audio) *Source {
 		service: audio.service,
 	}
 	if !isPhysicalDevice(sourceInfo.Name) {
-		masterSourceInfo := audio.getSourceInfoByName(sourceInfo.Proplist["device.master_device"])
-		if masterSourceInfo == nil {
-			logger.Warningf("cannot get master source for %s", sourceInfo.Name)
+		master := audio.getMasterNameFromVirtualDevice(sourceInfo.Name)
+		if master != "" {
+
+			masterSourceInfo := audio.getSourceInfoByName(master)
+			if masterSourceInfo == nil {
+				logger.Warningf("cannot get master source for %s", sourceInfo.Name)
+			} else {
+				sourceInfo.Card = masterSourceInfo.Card
+				sourceInfo.Ports = masterSourceInfo.Ports
+				sourceInfo.ActivePort = masterSourceInfo.ActivePort
+				logger.Debugf("create reducing noise source on %s", masterSourceInfo.Name)
+			}
 		} else {
-			sourceInfo.Card = masterSourceInfo.Card
-			sourceInfo.Ports = masterSourceInfo.Ports
-			sourceInfo.ActivePort = masterSourceInfo.ActivePort
-			logger.Debugf("create reducing noise source on %s", masterSourceInfo.Name)
+			logger.Warningf("cannot get master source for %s", sourceInfo.Name)
 		}
 	}
 
@@ -256,14 +262,19 @@ func (*Source) GetInterfaceName() string {
 func (s *Source) update(sourceInfo *pulse.Source) {
 	// 如果是虚拟通道，则将card和ports等设为对应主通道的值，这是为了能够正常使用降噪的训通道
 	if !isPhysicalDevice(sourceInfo.Name) {
-		masterSourceInfo := s.audio.getSourceInfoByName(sourceInfo.Proplist["device.master_device"])
-		if masterSourceInfo == nil {
-			logger.Warningf("cannot get master source for %s", sourceInfo.Name)
+		master := s.audio.getMasterNameFromVirtualDevice(sourceInfo.Name)
+		if master != "" {
+			masterSourceInfo := s.audio.getSourceInfoByName(master)
+			if masterSourceInfo == nil {
+				logger.Warningf("cannot get master source for %s", sourceInfo.Name)
+			} else {
+				sourceInfo.Card = masterSourceInfo.Card
+				sourceInfo.Ports = masterSourceInfo.Ports
+				sourceInfo.ActivePort = masterSourceInfo.ActivePort
+				logger.Debugf("create reducing noise source on %s", masterSourceInfo.Name)
+			}
 		} else {
-			sourceInfo.Card = masterSourceInfo.Card
-			sourceInfo.Ports = masterSourceInfo.Ports
-			sourceInfo.ActivePort = masterSourceInfo.ActivePort
-			logger.Debugf("create reducing noise source on %s", masterSourceInfo.Name)
+			logger.Warningf("cannot get master source for %s", sourceInfo.Name)
 		}
 	}
 
